@@ -37,7 +37,7 @@
 
 Data::Data() :
     num_rows(0), num_rows_rounded(0), num_cols(0), sparse_data(0), num_cols_no_sparse(0), externalData(true), index_data(
-        0) {
+        0), max_num_unique_values(0) {
 }
 
 Data::~Data() {
@@ -182,7 +182,22 @@ void Data::getAllValues(std::vector<double>& all_values, std::vector<size_t>& sa
     // If GWA data just use 0, 1, 2
     all_values = std::vector<double>( { 0, 1, 2 });
   }
+}
 
+void Data::getMinMaxValues(double& min, double&max, std::vector<size_t>& sampleIDs, size_t varID) {
+  if (sampleIDs.size() > 0) {
+    min = get(sampleIDs[0], varID);
+    max = min;
+  }
+  for (size_t i = 1; i < sampleIDs.size(); ++i) {
+    double value = get(sampleIDs[i], varID);
+    if (value < min) {
+      min = value;
+    }
+    if (value > max) {
+      max = value;
+    }
+  }
 }
 
 void Data::sort() {
@@ -209,5 +224,8 @@ void Data::sort() {
 
     // Save unique values
     unique_data_values.push_back(unique_values);
+    if (unique_values.size() > max_num_unique_values) {
+      max_num_unique_values = unique_values.size();
+    }
   }
 }
