@@ -47,9 +47,9 @@ Forest::Forest() :
     verbose_out(0), num_trees(DEFAULT_NUM_TREE), mtry(0), min_node_size(0), num_variables(0), num_independent_variables(
         0), seed(0), dependent_varID(0), num_samples(0), prediction_mode(false), memory_mode(MEM_DOUBLE), sample_with_replacement(
         true), memory_saving_splitting(false), splitrule(DEFAULT_SPLITRULE), predict_all(false), keep_inbag(false), sample_fraction(
-        1), holdout(false), prediction_type(DEFAULT_PREDICTIONTYPE), alpha(DEFAULT_ALPHA), minprop(DEFAULT_MINPROP), num_threads(
-        DEFAULT_NUM_THREADS), data(0), overall_prediction_error(0), importance_mode(DEFAULT_IMPORTANCE_MODE), progress(
-        0) {
+        1), holdout(false), prediction_type(DEFAULT_PREDICTIONTYPE), num_random_splits(DEFAULT_NUM_RANDOM_SPLITS), alpha(
+        DEFAULT_ALPHA), minprop(DEFAULT_MINPROP), num_threads(DEFAULT_NUM_THREADS), data(0), overall_prediction_error(
+        0), importance_mode(DEFAULT_IMPORTANCE_MODE), progress(0) {
 }
 
 Forest::~Forest() {
@@ -64,7 +64,8 @@ void Forest::initCpp(std::string dependent_variable_name, MemoryMode memory_mode
     std::string split_select_weights_file, std::vector<std::string>& always_split_variable_names,
     std::string status_variable_name, bool sample_with_replacement, std::vector<std::string>& unordered_variable_names,
     bool memory_saving_splitting, SplitRule splitrule, std::string case_weights_file, bool predict_all,
-    double sample_fraction, double alpha, double minprop, bool holdout, PredictionType prediction_type) {
+    double sample_fraction, double alpha, double minprop, bool holdout, PredictionType prediction_type,
+    uint num_random_splits) {
 
   this->verbose_out = verbose_out;
 
@@ -98,7 +99,8 @@ void Forest::initCpp(std::string dependent_variable_name, MemoryMode memory_mode
   // Call other init function
   init(dependent_variable_name, memory_mode, data, mtry, output_prefix, num_trees, seed, num_threads, importance_mode,
       min_node_size, status_variable_name, prediction_mode, sample_with_replacement, unordered_variable_names,
-      memory_saving_splitting, splitrule, predict_all, sample_fraction, alpha, minprop, holdout, prediction_type);
+      memory_saving_splitting, splitrule, predict_all, sample_fraction, alpha, minprop, holdout, prediction_type,
+      num_random_splits);
 
   if (prediction_mode) {
     loadFromFile(load_forest_filename);
@@ -154,14 +156,15 @@ void Forest::initR(std::string dependent_variable_name, Data* input_data, uint m
     std::string status_variable_name, bool prediction_mode, bool sample_with_replacement,
     std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
     std::vector<double>& case_weights, bool predict_all, bool keep_inbag, double sample_fraction, double alpha,
-    double minprop, bool holdout, PredictionType prediction_type) {
+    double minprop, bool holdout, PredictionType prediction_type, uint num_random_splits) {
 
   this->verbose_out = verbose_out;
 
   // Call other init function
   init(dependent_variable_name, MEM_DOUBLE, input_data, mtry, "", num_trees, seed, num_threads, importance_mode,
       min_node_size, status_variable_name, prediction_mode, sample_with_replacement, unordered_variable_names,
-      memory_saving_splitting, splitrule, predict_all, sample_fraction, alpha, minprop, holdout, prediction_type);
+      memory_saving_splitting, splitrule, predict_all, sample_fraction, alpha, minprop, holdout, prediction_type,
+      num_random_splits);
 
   // Set variables to be always considered for splitting
   if (!always_split_variable_names.empty()) {
@@ -190,7 +193,7 @@ void Forest::init(std::string dependent_variable_name, MemoryMode memory_mode, D
     uint min_node_size, std::string status_variable_name, bool prediction_mode, bool sample_with_replacement,
     std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
     bool predict_all, double sample_fraction, double alpha, double minprop, bool holdout,
-    PredictionType prediction_type) {
+    PredictionType prediction_type, uint num_random_splits) {
 
   // Initialize data with memmode
   this->data = input_data;
@@ -232,6 +235,7 @@ void Forest::init(std::string dependent_variable_name, MemoryMode memory_mode, D
   this->alpha = alpha;
   this->minprop = minprop;
   this->prediction_type = prediction_type;
+  this->num_random_splits = num_random_splits;
 
   // Set number of samples and variables
   num_samples = data->getNumRows();
@@ -425,7 +429,7 @@ void Forest::grow() {
     trees[i]->init(data, mtry, dependent_varID, num_samples, tree_seed, &deterministic_varIDs, &split_select_varIDs,
         tree_split_select_weights, importance_mode, min_node_size, &no_split_variables, sample_with_replacement,
         &is_ordered_variable, memory_saving_splitting, splitrule, &case_weights, keep_inbag, sample_fraction, alpha,
-        minprop, holdout);
+        minprop, holdout, num_random_splits);
   }
 
 // Init variable importance
