@@ -57,20 +57,20 @@ public:
       std::string status_variable_name, bool sample_with_replacement,
       std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
       std::string case_weights_file, bool predict_all, double sample_fraction, double alpha, double minprop,
-      bool holdout, PredictionType prediction_type);
+      bool holdout, PredictionType prediction_type, uint num_random_splits);
   void initR(std::string dependent_variable_name, Data* input_data, uint mtry, uint num_trees,
       std::ostream* verbose_out, uint seed, uint num_threads, ImportanceMode importance_mode, uint min_node_size,
       std::vector<std::vector<double>>& split_select_weights, std::vector<std::string>& always_split_variable_names,
       std::string status_variable_name, bool prediction_mode, bool sample_with_replacement,
       std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
       std::vector<double>& case_weights, bool predict_all, bool keep_inbag, double sample_fraction, double alpha,
-      double minprop, bool holdout, PredictionType prediction_type);
+      double minprop, bool holdout, PredictionType prediction_type, uint num_random_splits);
   void init(std::string dependent_variable_name, MemoryMode memory_mode, Data* input_data, uint mtry,
       std::string output_prefix, uint num_trees, uint seed, uint num_threads, ImportanceMode importance_mode,
       uint min_node_size, std::string status_variable_name, bool prediction_mode, bool sample_with_replacement,
       std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
       bool predict_all, double sample_fraction, double alpha, double minprop, bool holdout,
-      PredictionType prediction_type);
+      PredictionType prediction_type, uint num_random_splits);
   virtual void initInternal(std::string status_variable_name) = 0;
 
   // Grow or predict
@@ -114,7 +114,7 @@ public:
   double getOverallPredictionError() const {
     return overall_prediction_error;
   }
-  const std::vector<std::vector<double> >& getPredictions() const {
+  const std::vector<std::vector<std::vector<double>> >& getPredictions() const {
     return predictions;
   }
   size_t getDependentVarId() const {
@@ -202,6 +202,7 @@ protected:
   double sample_fraction;
   bool holdout;
   PredictionType prediction_type;
+  uint num_random_splits;
 
   // MAXSTAT splitrule
   double alpha;
@@ -224,7 +225,7 @@ protected:
   std::vector<Tree*> trees;
   Data* data;
 
-  std::vector<std::vector<double>> predictions;
+  std::vector<std::vector<std::vector<double>>> predictions;
   double overall_prediction_error;
 
   // Weight vector for selecting possible split variables, one weight between 0 (never select) and 1 (always select) for each variable
