@@ -51,7 +51,7 @@ public:
       std::vector<double>* split_select_weights, ImportanceMode importance_mode, uint min_node_size,
       std::vector<size_t>* no_split_variables, bool sample_with_replacement, std::vector<bool>* is_unordered,
       bool memory_saving_splitting, SplitRule splitrule, std::vector<double>* case_weights, bool keep_inbag,
-      double sample_fraction, double alpha, double minprop, bool holdout);
+      double sample_fraction, double alpha, double minprop, bool holdout, uint num_random_splits);
 
   virtual void initInternal() = 0;
 
@@ -178,6 +178,7 @@ protected:
   SplitRule splitrule;
   double alpha;
   double minprop;
+  uint num_random_splits;
 
 private:
   DISALLOW_COPY_AND_ASSIGN(Tree);
