@@ -56,9 +56,15 @@ void TreeClassification::initInternal() {
   // Init counters if not in memory efficient mode
   if (!memory_saving_splitting) {
     size_t num_classes = class_values->size();
-    size_t max_num_unique_values = data->getMaxNumUniqueValues();
-    counter = new size_t[max_num_unique_values];
-    counter_per_class = new size_t[num_classes * max_num_unique_values];
+    size_t max_num_splits = data->getMaxNumUniqueValues();
+
+    // Use number of random splits for extratrees
+    if (splitrule == EXTRATREES && num_random_splits > max_num_splits) {
+      max_num_splits = num_random_splits;
+    }
+
+    counter = new size_t[max_num_splits];
+    counter_per_class = new size_t[num_classes * max_num_splits];
   }
 }
 
@@ -102,8 +108,14 @@ bool TreeClassification::splitNodeInternal(size_t nodeID, std::vector<size_t>& p
     return true;
   }
 
-// Find best split, stop if no decrease of impurity
-  bool stop = findBestSplit(nodeID, possible_split_varIDs);
+  // Find best split, stop if no decrease of impurity
+  bool stop;
+  if (splitrule == EXTRATREES) {
+    stop = findBestSplitExtraTrees(nodeID, possible_split_varIDs);
+  } else {
+    stop = findBestSplit(nodeID, possible_split_varIDs);
+  }
+
   if (stop) {
     split_values[nodeID] = estimate(nodeID);
     return true;
@@ -203,11 +215,9 @@ void TreeClassification::findBestSplitValueSmallQ(size_t nodeID, size_t varID, s
     return;
   }
 
-  // Remove largest value because no split possible
-  possible_split_values.pop_back();
-
   // Initialize with 0, if not in memory efficient mode, use pre-allocated space
-  size_t num_splits = possible_split_values.size();
+  // -1 because no split possible at largest value
+  size_t num_splits = possible_split_values.size() - 1;
   size_t* class_counts_right;
   size_t* n_right;
   if (memory_saving_splitting) {
@@ -261,15 +271,15 @@ void TreeClassification::findBestSplitValueSmallQ(size_t nodeID, size_t varID, s
 
     // If better than before, use this
     if (decrease > best_decrease) {
-      best_value = possible_split_values[i];
+      best_value =  (possible_split_values[i] + possible_split_values[i + 1]) / 2;
       best_varID = varID;
       best_decrease = decrease;
     }
   }
 
-    if (memory_saving_splitting) {
-      delete[] class_counts_right;
-      delete[] n_right;
+  if (memory_saving_splitting) {
+    delete[] class_counts_right;
+    delete[] n_right;
   }
 }
 
@@ -325,7 +335,14 @@ void TreeClassification::findBestSplitValueLargeQ(size_t nodeID, size_t varID, s
 
     // If better than before, use this
     if (decrease > best_decrease) {
-      best_value = data->getUniqueDataValue(varID, i);
+      // Find next value in this node
+      size_t j = i + 1;
+      while(j < num_unique && counter[j] == 0) {
+        ++j;
+      }
+
+      // Use mid-point split
+      best_value = (data->getUniqueDataValue(varID, i) + data->getUniqueDataValue(varID, j)) / 2;
       best_varID = varID;
       best_decrease = decrease;
     }
@@ -408,6 +425,242 @@ void TreeClassification::findBestSplitValueUnordered(size_t nodeID, size_t varID
   }
 }
 
+bool TreeClassification::findBestSplitExtraTrees(size_t nodeID, std::vector<size_t>& possible_split_varIDs) {
+
+  size_t num_samples_node = sampleIDs[nodeID].size();
+  size_t num_classes = class_values->size();
+  double best_decrease = -1;
+  size_t best_varID = 0;
+  double best_value = 0;
+
+  size_t* class_counts = new size_t[num_classes]();
+  // Compute overall class counts
+  for (size_t i = 0; i < num_samples_node; ++i) {
+    size_t sampleID = sampleIDs[nodeID][i];
+    uint sample_classID = (*response_classIDs)[sampleID];
+    ++class_counts[sample_classID];
+  }
+
+  // For all possible split variables
+  for (auto& varID : possible_split_varIDs) {
+    // Find best split value, if ordered consider all values as split values, else all 2-partitions
+    if ((*is_ordered_variable)[varID]) {
+      findBestSplitValueExtraTrees(nodeID, varID, num_classes, class_counts, num_samples_node, best_value, best_varID,
+          best_decrease);
+    } else {
+      findBestSplitValueExtraTreesUnordered(nodeID, varID, num_classes, class_counts, num_samples_node, best_value,
+          best_varID, best_decrease);
+    }
+  }
+
+  delete[] class_counts;
+
+  // Stop if no good split found
+  if (best_decrease < 0) {
+    return true;
+  }
+
+  // Save best values
+  split_varIDs[nodeID] = best_varID;
+  split_values[nodeID] = best_value;
+
+  // Compute gini index for this node and to variable importance if needed
+  if (importance_mode == IMP_GINI) {
+    addGiniImportance(nodeID, best_varID, best_decrease);
+  }
+  return false;
+}
+
+void TreeClassification::findBestSplitValueExtraTrees(size_t nodeID, size_t varID, size_t num_classes,
+    size_t* class_counts, size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease) {
+
+  // Get min/max values of covariate in node
+  double min;
+  double max;
+  data->getMinMaxValues(min, max, sampleIDs[nodeID], varID);
+
+  // Try next variable if all equal for this
+  if (min == max) {
+    return;
+  }
+
+  // Create possible split values: Draw randomly between min and max
+  std::vector<double> possible_split_values;
+  std::uniform_real_distribution<double> udist(min, max);
+  possible_split_values.reserve(num_random_splits);
+  for (size_t i = 0; i < num_random_splits; ++i) {
+    possible_split_values.push_back(udist(random_number_generator));
+  }
+
+  // Initialize with 0, if not in memory efficient mode, use pre-allocated space
+  size_t num_splits = possible_split_values.size();
+  size_t* class_counts_right;
+  size_t* n_right;
+  if (memory_saving_splitting) {
+    class_counts_right = new size_t[num_splits * num_classes]();
+    n_right = new size_t[num_splits]();
+  } else {
+    class_counts_right = counter_per_class;
+    n_right = counter;
+    std::fill(class_counts_right, class_counts_right + num_splits * num_classes, 0);
+    std::fill(n_right, n_right + num_splits, 0);
+  }
+
+  // Count samples in right child per class and possbile split
+  for (auto& sampleID : sampleIDs[nodeID]) {
+    double value = data->get(sampleID, varID);
+    uint sample_classID = (*response_classIDs)[sampleID];
+
+    // Count samples until split_value reached
+    for (size_t i = 0; i < num_splits; ++i) {
+      if (value > possible_split_values[i]) {
+        ++n_right[i];
+        ++class_counts_right[i * num_classes + sample_classID];
+      } else {
+        break;
+      }
+    }
+  }
+
+  // Compute decrease of impurity for each possible split
+  for (size_t i = 0; i < num_splits; ++i) {
+
+    // Stop if one child empty
+    size_t n_left = num_samples_node - n_right[i];
+    if (n_left == 0 || n_right[i] == 0) {
+      continue;
+    }
+
+    // Sum of squares
+    double sum_left = 0;
+    double sum_right = 0;
+    for (size_t j = 0; j < num_classes; ++j) {
+      size_t class_count_right = class_counts_right[i * num_classes + j];
+      size_t class_count_left = class_counts[j] - class_count_right;
+
+      sum_right += class_count_right * class_count_right;
+      sum_left += class_count_left * class_count_left;
+    }
+
+    // Decrease of impurity
+    double decrease = sum_left / (double) n_left + sum_right / (double) n_right[i];
+
+    // If better than before, use this
+    if (decrease > best_decrease) {
+      best_value = possible_split_values[i];
+      best_varID = varID;
+      best_decrease = decrease;
+    }
+  }
+
+  if (memory_saving_splitting) {
+    delete[] class_counts_right;
+    delete[] n_right;
+  }
+}
+
+void TreeClassification::findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t varID, size_t num_classes,
+    size_t* class_counts, size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease) {
+
+  size_t num_unique_values = data->getNumUniqueDataValues(varID);
+
+  // Get all factor indices in node
+  std::vector<bool> factor_in_node(num_unique_values, false);
+  for (auto& sampleID : sampleIDs[nodeID]) {
+    size_t index = data->getIndex(sampleID, varID);
+    factor_in_node[index] = true;
+  }
+
+  // Vector of indices in and out of node
+  std::vector<size_t> indices_in_node;
+  std::vector<size_t> indices_out_node;
+  indices_in_node.reserve(num_unique_values);
+  indices_out_node.reserve(num_unique_values);
+  for (size_t i = 0; i < num_unique_values; ++i) {
+    if (factor_in_node[i]) {
+      indices_in_node.push_back(i);
+    } else {
+      indices_out_node.push_back(i);
+    }
+  }
+
+  // Generate num_random_splits splits
+  for (size_t i = 0; i < num_random_splits; ++i) {
+    std::vector<size_t> split_subset;
+    split_subset.reserve(num_unique_values);
+
+    // Draw random subsets, sample all partitions with equal probability
+    if (indices_in_node.size() > 1) {
+      size_t num_partitions = (2 << (indices_in_node.size() - 1)) - 2; // 2^n-2 (don't allow full or empty)
+      std::uniform_int_distribution<size_t> udist(1, num_partitions);
+      size_t splitID_in_node = udist(random_number_generator);
+      for (size_t j = 0; j < indices_in_node.size(); ++j) {
+        if ((splitID_in_node & (1 << j)) > 0) {
+          split_subset.push_back(indices_in_node[j]);
+        }
+      }
+    }
+    if (indices_out_node.size() > 1) {
+      size_t num_partitions = (2 << (indices_out_node.size() - 1)) - 1; // 2^n-1 (allow full or empty)
+      std::uniform_int_distribution<size_t> udist(0, num_partitions);
+      size_t splitID_out_node = udist(random_number_generator);
+      for (size_t j = 0; j < indices_out_node.size(); ++j) {
+        if ((splitID_out_node & (1 << j)) > 0) {
+          split_subset.push_back(indices_out_node[j]);
+        }
+      }
+    }
+
+    // Assign union of the two subsets to right child
+    size_t splitID = 0;
+    for (auto& idx : split_subset) {
+      splitID |= 1 << idx;
+    }
+
+    // Initialize
+    size_t* class_counts_right = new size_t[num_classes]();
+    size_t n_right = 0;
+
+    // Count classes in left and right child
+    for (auto& sampleID : sampleIDs[nodeID]) {
+      uint sample_classID = (*response_classIDs)[sampleID];
+      double value = data->get(sampleID, varID);
+      size_t factorID = floor(value) - 1;
+
+      // If in right child, count
+      // In right child, if bitwise splitID at position factorID is 1
+      if ((splitID & (1 << factorID))) {
+        ++n_right;
+        ++class_counts_right[sample_classID];
+      }
+    }
+    size_t n_left = num_samples_node - n_right;
+
+    // Sum of squares
+    double sum_left = 0;
+    double sum_right = 0;
+    for (size_t j = 0; j < num_classes; ++j) {
+      size_t class_count_right = class_counts_right[j];
+      size_t class_count_left = class_counts[j] - class_count_right;
+
+      sum_right += class_count_right * class_count_right;
+      sum_left += class_count_left * class_count_left;
+    }
+
+    // Decrease of impurity
+    double decrease = sum_left / (double) n_left + sum_right / (double) n_right;
+
+    // If better than before, use this
+    if (decrease > best_decrease) {
+      best_value = splitID;
+      best_varID = varID;
+      best_decrease = decrease;
+    }
+
+    delete[] class_counts_right;
+  }
+}
+
 void TreeClassification::addGiniImportance(size_t nodeID, size_t varID, double decrease) {
 
   std::vector<size_t> class_counts;
