@@ -73,6 +73,12 @@ private:
   void findBestSplitValueUnordered(size_t nodeID, size_t varID, size_t num_classes, size_t* class_counts,
       size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease);
 
+  bool findBestSplitExtraTrees(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
+  void findBestSplitValueExtraTrees(size_t nodeID, size_t varID, size_t num_classes,
+      size_t* class_counts, size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease);
+  void findBestSplitValueExtraTreesUnordered(size_t nodeID, size_t varID, size_t num_classes,
+      size_t* class_counts, size_t num_samples_node, double& best_value, size_t& best_varID, double& best_decrease);
+
   void addGiniImportance(size_t nodeID, size_t varID, double decrease);
 
   void cleanUpInternal() {
