@@ -51,7 +51,8 @@ Rcpp::List rangerCpp(uint treetype, std::string dependent_variable_name,
     bool sample_with_replacement, bool probability, std::vector<std::string>& unordered_variable_names,
     bool use_unordered_variable_names, bool save_memory, uint splitrule_r, 
     std::vector<double>& case_weights, bool use_case_weights, bool predict_all, 
-    bool keep_inbag, double sample_fraction, double alpha, double minprop, bool holdout, uint prediction_type_r) {
+    bool keep_inbag, double sample_fraction, double alpha, double minprop, bool holdout, uint prediction_type_r, 
+    uint num_random_splits) {
 
   Rcpp::List result;
   Forest* forest = 0;
@@ -117,7 +118,7 @@ Rcpp::List rangerCpp(uint treetype, std::string dependent_variable_name,
     forest->initR(dependent_variable_name, data, mtry, num_trees, verbose_out, seed, num_threads,
         importance_mode, min_node_size, split_select_weights, always_split_variable_names, status_variable_name,
         prediction_mode, sample_with_replacement, unordered_variable_names, save_memory, splitrule, case_weights, 
-        predict_all, keep_inbag, sample_fraction, alpha, minprop, holdout, prediction_type);
+        predict_all, keep_inbag, sample_fraction, alpha, minprop, holdout, prediction_type, num_random_splits);
 
     // Load forest object if in prediction mode
     if (prediction_mode) {
@@ -158,11 +159,22 @@ Rcpp::List rangerCpp(uint treetype, std::string dependent_variable_name,
           << "Warning: Split select weights used. Variable importance measures are only comparable for variables with equal weights."
           << std::endl;
     }
-
+    
+    // Use first non-empty dimension of predictions
+    const std::vector<std::vector<std::vector<double>>>& predictions = forest->getPredictions();
+    if (predictions.size() == 1) {
+      if (predictions[0].size() == 1) {
+        result.push_back(forest->getPredictions()[0][0], "predictions");
+      } else {
+        result.push_back(forest->getPredictions()[0], "predictions");
+      }
+    } else {
+      result.push_back(forest->getPredictions(), "predictions");
+    }
+    
     // Return output
     result.push_back(forest->getNumTrees(), "num.trees");
     result.push_back(forest->getNumIndependentVariables(), "num.independent.variables");
-    result.push_back(forest->getPredictions(), "predictions");
     if (treetype == TREE_SURVIVAL) {
       ForestSurvival* temp = (ForestSurvival*) forest;
       result.push_back(temp->getUniqueTimepoints(), "unique.death.times");
