@@ -98,10 +98,11 @@ void loadDoubleVectorFromFile(std::vector<double>& result, std::string filename)
 
 void drawWithoutReplacementSkip(std::vector<size_t>& result, std::mt19937_64& random_number_generator, size_t max,
     std::vector<size_t>& skip, size_t num_samples) {
-  if (num_samples < max / 2) {
+  if (num_samples < max / 10) {
     drawWithoutReplacementSimple(result, random_number_generator, max, skip, num_samples);
   } else {
-    drawWithoutReplacementKnuth(result, random_number_generator, max, skip, num_samples);
+    //drawWithoutReplacementKnuth(result, random_number_generator, max, skip, num_samples);
+    drawWithoutReplacementFisherYates(result, random_number_generator, max, skip, num_samples);
   }
 }
 
@@ -161,6 +162,28 @@ void drawWithoutReplacementKnuth(std::vector<size_t>& result, std::mt19937_64& r
   }
 }
 
+void drawWithoutReplacementFisherYates(std::vector<size_t>& result, std::mt19937_64& random_number_generator,
+    size_t max, std::vector<size_t>& skip, size_t num_samples) {
+
+  // Create indices
+  result.resize(max);
+  std::iota(result.begin(), result.end(), 0);
+
+   // Skip indices
+  for (size_t i = 0; i < skip.size(); ++i) {
+    result.erase(result.begin() + skip[skip.size() - 1 - i]);
+  }
+
+  // Draw without replacement using Fisher Yates algorithm
+  std::uniform_real_distribution<double> distribution(0.0, 1.0);
+  for (size_t i = 0; i < num_samples; ++i) {
+    size_t j = i + distribution(random_number_generator) * (max - skip.size() - i);
+    std::swap(result[i], result[j]);
+  }
+
+  result.resize(num_samples);
+}
+
 void drawWithoutReplacementWeighted(std::vector<size_t>& result, std::mt19937_64& random_number_generator,
     std::vector<size_t>& indices, size_t num_samples, std::vector<double>& weights) {
 
@@ -323,7 +346,7 @@ size_t roundToNextMultiple(size_t value, uint multiple) {
     return value;
   }
 
-  int remainder = value % multiple;
+  size_t remainder = value % multiple;
   if (remainder == 0) {
     return value;
   }
@@ -427,6 +450,10 @@ double maxstatPValueLau94(double b, double minprop, double maxprop, size_t N, st
   return 2 * (1 - pstdnorm(b)) + D;
 }
 
+double maxstatPValueUnadjusted(double b) {
+  return 2 * pstdnorm(-b);
+}
+
 double dstdnorm(double x) {
   return exp(-0.5 * x * x) / sqrt(2 * M_PI);
 }
@@ -594,7 +621,13 @@ void maxstat(std::vector<double>& scores, std::vector<double>& x, std::vector<si
 
     if (T > best_maxstat) {
       best_maxstat = T;
-      best_split_value = x[indices[i]];
+
+      // Use mid-point split if possible
+      if (i < n - 1) {
+        best_split_value = (x[indices[i]] + x[indices[i + 1]]) / 2;
+      } else {
+        best_split_value = x[indices[i]];
+      }
     }
   }
 }
