@@ -64,6 +64,11 @@ test_that("Importance p-values Janitza-Holdout: returns correct dimensions", {
   expect_equal(dim(vimp), c(104, 2))
 })
 
+test_that("Importance p-values Janitza-Holdout: No zero p-values", {
+  expect_warning(vimp <- importance_pvalues(holdout_p100, method = "janitza"))
+  expect_false(any(vimp[, "pvalue"] == 0))
+})
+
 ## Altmann
 test_that("Importance p-values Altmann: returns correct dimensions", {
   vimp <- importance_pvalues(rf_p0, method = "altmann", formula = Species ~ ., data = iris)
@@ -80,6 +85,11 @@ test_that("Importance p-values Altmann: not working for holdoutRF", {
   expect_error(importance_pvalues(holdout_p0, method = "altmann", formula = Species ~ ., data = iris))
 })
 
+test_that("Importance p-values Janitza-Holdout: No zero p-values", {
+  vimp <- importance_pvalues(rf_p0, method = "altmann", formula = Species ~ ., data = iris)
+  expect_false(any(vimp[, "pvalue"] == 0))
+})
+
 ## Hold-out RF
 test_that("HoldoutRF working", {
   expect_is(holdout_p0, "holdoutRF")
@@ -94,3 +104,35 @@ test_that("HoldoutRF ... argument working", {
   rf <- holdoutRF(Species ~., iris, num.trees = 10)
   expect_equal(rf$rf1$num.trees, 10)
 })
+
+test_that("HoldoutRF working with formula", {
+  rf <- holdoutRF(Species ~., iris, num.trees = 10)
+  expect_equal(rf$rf1$treetype, "Classification")
+  
+  rf <- holdoutRF(Species ~., data = iris, num.trees = 10)
+  expect_equal(rf$rf1$treetype, "Classification")
+  
+  rf <- holdoutRF(formula = Species ~., iris, num.trees = 10)
+  expect_equal(rf$rf1$treetype, "Classification")
+  
+  rf <- holdoutRF(data = iris, formula = Species ~., num.trees = 10)
+  expect_equal(rf$rf1$treetype, "Classification")
+})
+
+test_that("HoldoutRF working with dependent.variable.name", {
+  rf <- holdoutRF(dependent.variable.name = "Species", data = iris, num.trees = 10)
+  expect_equal(rf$rf1$treetype, "Classification")
+  
+  rf <- holdoutRF(data = iris, dependent.variable.name = "Species", num.trees = 10)
+  expect_equal(rf$rf1$treetype, "Classification")
+})
+
+test_that("HoldoutRF not working if importance argument used", {
+  expect_error(holdoutRF(Species ~., iris, num.trees = 10, importance = "impurity"), 
+               "Error: Argument 'importance' not supported in holdoutRF.")
+})
+
+test_that("HoldoutRF not working if replace argument used", {
+  expect_error(holdoutRF(Species ~., iris, num.trees = 10, replace = TRUE), 
+               "Error: Argument 'replace' not supported in holdoutRF.")
+})
